@@ -9,8 +9,11 @@ from functools import lru_cache
 from typing import List, Any, Tuple
 
 import caffe
+import hashlib
 import numpy as np
 
+from spacer import config
+
 
 class Transformer:
     """
@@ -70,19 +73,19 @@ def classify_from_imlist(im_list: List,
     :param startlayer: name of first convolutional layer.
     :return: features list.
     """
+    with config.log_entry_and_exit('forward pass through net'):
+        scorelist = []
+        for b in range(len(im_list) // batch_size + 1):
+            for i in range(batch_size):
+                pos = b * batch_size + i
+                if pos < len(im_list):
+                    net.blobs['data'].data[i, :, :, :] = \
+                        transformer.preprocess(im_list[pos])
+            net.forward(start=startlayer)
+            scorelist.extend(list(copy(net.blobs[scorelayer].data).
+                                  astype(np.float)))
 
-    scorelist = []
-    for b in range(len(im_list) // batch_size + 1):
-        for i in range(batch_size):
-            pos = b * batch_size + i
-            if pos < len(im_list):
-                net.blobs['data'].data[i, :, :, :] = \
-                    transformer.preprocess(im_list[pos])
-        net.forward(start=startlayer)
-        scorelist.extend(list(copy(net.blobs[scorelayer].data).
-                              astype(np.float)))
-
-    scorelist = scorelist[:len(im_list)]
+        scorelist = scorelist[:len(im_list)]
 
     return scorelist
 
@@ -96,6 +99,12 @@ def load_net(modeldef_path: str,
     :param modelweighs_path: pretrained weights path.
     :return: pretrained model.
     """
+    # To verify that the correct weight is loaded
+    with config.log_entry_and_exit("load net, check SHA"):
+        with open(modelweighs_path, 'rb') as fp:
+            sha256 = hashlib.sha256(fp.read()).hexdigest()
+        assert sha256 == config.MODEL_WEIGHTS_SHA['vgg16']
+
     return caffe.Net(modeldef_path, modelweighs_path, caffe.TEST)
 
 
